@@ -1,8 +1,8 @@
-/* $Id: Configuration.java,v 1.17 2005/06/11 13:13:15 eric Exp $
+/* $Id: Configuration.java,v 1.17.2.3 2007/01/18 21:31:51 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -153,6 +153,11 @@ public class Configuration
      */
     public boolean   overloadAggressively        = false;
 
+    /**
+     * Specifies whether to generate globally unique class member names.
+     */
+    public boolean   useUniqueClassMemberNames   = false;
+
     /**
      * An optional default package to which all classes whose name is obfuscated
      * can be moved.
