@@ -1,8 +1,8 @@
-/* $Id: ProGuardTask.java,v 1.32 2005/06/11 13:21:35 eric Exp $
+/* $Id: ProGuardTask.java,v 1.32.2.3 2007/01/18 21:31:51 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -142,6 +142,12 @@ public class ProGuardTask extends ConfigurationTask
     }
 
 
+    public void setUseuniqueclassmembernames(boolean useUniqueClassMemberNames)
+    {
+        configuration.useUniqueClassMemberNames = useUniqueClassMemberNames;
+    }
+
+
     public void setDefaultpackage(String defaultPackage)
     {
         configuration.defaultPackage = ClassUtil.internalClassName(defaultPackage);
