@@ -1,9 +1,9 @@
-/* $Id: LibraryFieldInfo.java,v 1.16 2005/06/25 22:07:51 eric Exp $
+/* $Id: LibraryFieldInfo.java,v 1.16.2.3 2007/01/18 21:31:51 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
  * Copyright (c) 1999      Mark Welsh (markw@retrologic.com)
- * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -60,11 +60,21 @@ public class LibraryFieldInfo extends LibraryMemberInfo implements FieldInfo
     }
 
 
-    /**
-     * Accepts the given visitor.
-     */
+    // Implementations for LibraryMemberInfo.
+
     public void accept(LibraryClassFile libraryClassFile, MemberInfoVisitor memberInfoVisitor)
     {
         memberInfoVisitor.visitLibraryFieldInfo(libraryClassFile, this);
     }
+
+
+    // Implementations for MemberInfo.
+
+    public void referencedClassesAccept(ClassFileVisitor classFileVisitor)
+    {
+        if (referencedClassFile != null)
+        {
+            referencedClassFile.accept(classFileVisitor);
+        }
+    }
 }
