@@ -1,9 +1,9 @@
-/* $Id: ProgramMemberInfo.java,v 1.29 2005/06/11 13:13:15 eric Exp $
+/* $Id: ProgramMemberInfo.java,v 1.29.2.3 2007/01/18 21:31:51 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
  * Copyright (c) 1999      Mark Welsh (markw@retrologic.com)
- * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -112,13 +112,6 @@ abstract public class ProgramMemberInfo implements MemberInfo
                                           AttrInfoVisitor  attrInfoVisitor);
 
 
-    /**
-     * Lets the ClassFile objects referenced in the descriptor string
-     * accept the given visitor.
-     */
-    public abstract void referencedClassesAccept(ClassFileVisitor classFileVisitor);
-
-
     /**
      * Imports the field or method data to internal representation.
      */
