@@ -1,8 +1,8 @@
-/* $Id: Annotation.java,v 1.4 2005/06/11 13:13:15 eric Exp $
+/* $Id: Annotation.java,v 1.4.2.3 2007/01/18 21:31:51 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
@@ -21,6 +21,7 @@
 package proguard.classfile.attribute.annotation;
 
 import proguard.classfile.*;
+import proguard.classfile.visitor.ClassFileVisitor;
 
 import java.io.*;
 
@@ -108,6 +109,42 @@ public class Annotation implements VisitorAccepter
     }
 
 
+    /**
+     * Applies the given visitor to the first referenced class. This is the
+     * main annotation class.
+     */
+    public void referencedClassFileAccept(ClassFileVisitor classFileVisitor)
+    {
+        if (referencedClassFiles != null)
+        {
+            ClassFile referencedClassFile = referencedClassFiles[0];
+            if (referencedClassFile != null)
+            {
+                referencedClassFile.accept(classFileVisitor);
+            }
+        }
+    }
+
+
+    /**
+     * Applies the given visitor to all referenced classes.
+     */
+    public void referencedClassFilesAccept(ClassFileVisitor classFileVisitor)
+    {
+        if (referencedClassFiles != null)
+        {
+            for (int index = 0; index < referencedClassFiles.length; index++)
+            {
+                ClassFile referencedClassFile = referencedClassFiles[index];
+                if (referencedClassFile != null)
+                {
+                    referencedClassFile.accept(classFileVisitor);
+                }
+            }
+        }
+    }
+
+
     /**
      * Applies the given visitor to all element value pairs.
      */
