@@ -1,8 +1,8 @@
-/* $Id: ElementValue.java,v 1.6 2005/06/11 13:13:15 eric Exp $
+/* $Id: ElementValue.java,v 1.6.2.3 2007/01/18 21:31:51 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
@@ -21,6 +21,7 @@
 package proguard.classfile.attribute.annotation;
 
 import proguard.classfile.*;
+import proguard.classfile.visitor.MemberInfoVisitor;
 
 import java.io.*;
 
@@ -42,10 +43,17 @@ public abstract class ElementValue implements VisitorAccepter
      */
     public int u2elementName;
 
+    /**
+     * An extra field pointing to the referenced ClassFile object.
+     * This field is typically filled out by the <code>{@link
+     * proguard.classfile.util.ClassReferenceInitializer}</code>.
+     */
+    public ClassFile referencedClassFile;
+
     /**
      * An extra field pointing to the referenced <code>MethodInfo</code>
      * object, if applicable. This field is typically filled out by the
-     * <code>{@link ClassFileReferenceInitializer}</code>.
+     * <code>{@link proguard.classfile.util.ClassFileReferenceInitializer}</code>.
      */
     public MethodInfo referencedMethodInfo;
 
@@ -141,6 +149,17 @@ public abstract class ElementValue implements VisitorAccepter
     public abstract void accept(ClassFile classFile, Annotation annotation, ElementValueVisitor elementValueVisitor);
 
 
+    /**
+     * Applies the given visitor to the referenced method.
+     */
+    public void referencedMethodInfoAccept(MemberInfoVisitor memberInfoVisitor)
+    {
+        if (referencedMethodInfo != null)
+        {
+            referencedMethodInfo.accept(referencedClassFile, memberInfoVisitor);
+        }
+    }
+
 
     // Implementations for VisitorAccepter.
 
