@@ -1,8 +1,8 @@
-/* $Id: BranchInstruction.java,v 1.15 2005/06/11 13:13:15 eric Exp $
+/* $Id: BranchInstruction.java,v 1.15.2.3 2007/01/18 21:31:51 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -65,9 +65,10 @@ public class BranchInstruction extends Instruction
 
     public Instruction shrink()
     {
-        // Is this a wide branch that can be replaced by a normal branch?
-        if (branchOffset << 16 >> 16 == branchOffset)
+        // Do we need an ordinary branch or a wide branch?
+        if (requiredBranchOffsetSize() == 2)
         {
+            // Can we replace the wide branch by an ordinary branch?
             if      (opcode == InstructionConstants.OP_GOTO_W)
             {
                 opcode = InstructionConstants.OP_GOTO;
@@ -77,6 +78,22 @@ public class BranchInstruction extends Instruction
                 opcode = InstructionConstants.OP_JSR;
             }
         }
+        else
+        {
+            // Can we replace the ordinary branch by a wide branch?
+            if      (opcode == InstructionConstants.OP_GOTO)
+            {
+                opcode = InstructionConstants.OP_GOTO_W;
+            }
+            else if (opcode == InstructionConstants.OP_JSR)
+            {
+                opcode = InstructionConstants.OP_JSR_W;
+            }
+            else
+            {
+                throw new IllegalArgumentException("Branch instruction can't be widened ("+this.toString()+")");
+            }
+        }
 
         return this;
     }
@@ -89,6 +106,11 @@ public class BranchInstruction extends Instruction
 
     protected void writeInfo(byte[] code, int offset)
     {
+        if (requiredBranchOffsetSize() > branchOffsetSize())
+        {
+            throw new IllegalArgumentException("Instruction has invalid branch offset size ("+this.toString(offset)+")");
+        }
+
         writeValue(code, offset, branchOffset, branchOffsetSize());
     }
 
@@ -122,7 +144,7 @@ public class BranchInstruction extends Instruction
     // Small utility methods.
 
     /**
-     * Computes the appropriate branch offset size for this instruction.
+     * Returns the branch offset size for this instruction.
      */
     private int branchOffsetSize()
     {
@@ -130,4 +152,15 @@ public class BranchInstruction extends Instruction
                opcode == InstructionConstants.OP_JSR_W  ? 4 :
                                                           2;
     }
+
+
+    /**
+     * Computes the required branch offset size for this instruction's branch
+     * offset.
+     */
+    private int requiredBranchOffsetSize()
+    {
+        return branchOffset << 16 >> 16 == branchOffset ? 2 :
+                                                          4;
+    }
 }
