@@ -1,8 +1,8 @@
-/* $Id: FilteredDataEntryReader.java,v 1.3 2005/06/11 13:13:15 eric Exp $
+/* $Id: FilteredDataEntryReader.java,v 1.3.2.3 2007/01/18 21:31:52 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -78,19 +78,13 @@ public class FilteredDataEntryReader implements DataEntryReader
     public void read(DataEntry dataEntry)
     throws IOException
     {
-        if (dataEntryFilter.accepts(dataEntry))
+        DataEntryReader dataEntryReader = dataEntryFilter.accepts(dataEntry) ?
+            acceptedDataEntryReader :
+            rejectedDataEntryReader;
+
+        if (dataEntryReader != null)
         {
-            if (acceptedDataEntryReader != null)
-            {
-                acceptedDataEntryReader.read(dataEntry);
-            }
-        }
-        else
-        {
-            if (rejectedDataEntryReader != null)
-            {
-                rejectedDataEntryReader.read(dataEntry);
-            }
+            dataEntryReader.read(dataEntry);
         }
     }
 }
