@@ -1,8 +1,8 @@
-/* $Id: MemberInfoObfuscator.java,v 1.14 2005/06/11 13:13:16 eric Exp $
+/* $Id: MemberInfoObfuscator.java,v 1.14.2.5 2007/01/18 21:31:52 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -21,7 +21,7 @@
 package proguard.obfuscate;
 
 import proguard.classfile.*;
-import proguard.classfile.util.MethodInfoLinker;
+import proguard.classfile.util.*;
 import proguard.classfile.visitor.MemberInfoVisitor;
 
 import java.util.*;
@@ -30,6 +30,7 @@ import java.util.*;
  * This MemberInfoVisitor obfuscates all class members that it visits.
  * It uses names from the given name factory. At the same time, it avoids names
  * from the given descriptor map.
+ * <p>
  * The class members must have been linked before applying this visitor.
  *
  * @see MethodInfoLinker
@@ -38,9 +39,9 @@ import java.util.*;
  */
 public class MemberInfoObfuscator implements MemberInfoVisitor
 {
-    private boolean     allowAggressiveOverloading;
-    private NameFactory nameFactory;
-    private Map         descriptorMap;
+    private boolean        allowAggressiveOverloading;
+    private NameFactory    nameFactory;
+    private Map            descriptorMap;
 
 
     /**
@@ -49,12 +50,12 @@ public class MemberInfoObfuscator implements MemberInfoVisitor
      *                                   members can be overloaded aggressively.
      * @param nameFactory                the factory that can produce
      *                                   obfuscated member names.
-     * @param descriptorMap              the map of names to avoid:
-     *                                   [member descriptor - new name - old name].
+     * @param descriptorMap              the map of descriptors to
+     *                                   [new name - old name] maps.
      */
-    public MemberInfoObfuscator(boolean     allowAggressiveOverloading,
-                                NameFactory nameFactory,
-                                Map         descriptorMap)
+    public MemberInfoObfuscator(boolean        allowAggressiveOverloading,
+                                NameFactory    nameFactory,
+                                Map            descriptorMap)
     {
         this.allowAggressiveOverloading = allowAggressiveOverloading;
         this.nameFactory                = nameFactory;
@@ -92,17 +93,18 @@ public class MemberInfoObfuscator implements MemberInfoVisitor
 
 
     /**
-     * Inserts the given class member into the main map.
+     * Obfuscates the given class member.
      * @param classFile  the class file of the given member.
-     * @param memberInfo the class member to be linked.
+     * @param memberInfo the class member to be obfuscated.
      */
-    private void visitMemberInfo(ClassFile classFile, MemberInfo memberInfo)
+    private void visitMemberInfo(ClassFile  classFile,
+                                 MemberInfo memberInfo)
     {
         // Get the member's name and descriptor.
         String name       = memberInfo.getName(classFile);
         String descriptor = memberInfo.getDescriptor(classFile);
 
-        // Check whether we're allowed to do aggressive overloading
+        // Check whether we're allowed to overload aggressively.
         if (!allowAggressiveOverloading)
         {
             // Trim the return argument from the descriptor if not.
@@ -110,9 +112,8 @@ public class MemberInfoObfuscator implements MemberInfoVisitor
             descriptor = descriptor.substring(0, descriptor.indexOf(')')+1);
         }
 
-        // Put the [descriptor - new name] in the map,
-        // creating a new [new name - old name] map if necessary.
-        Map newNameMap = retrieveNameMap(descriptorMap, descriptor);
+        // Get the name map, creating a new one if necessary.
+        Map nameMap = retrieveNameMap(descriptorMap, descriptor);
 
         // Get the member's new name.
         String newName = newMemberName(memberInfo);
@@ -127,26 +128,13 @@ public class MemberInfoObfuscator implements MemberInfoVisitor
             {
                 newName = nameFactory.nextName();
             }
-            while (newNameMap.containsKey(newName));
+            while (nameMap.containsKey(newName));
+
+            // Remember not to use the new name again in this name space.
+            nameMap.put(newName, name);
 
             // Assign the new name.
             setNewMemberName(memberInfo, newName);
-
-            // Remember not to use the new name again in this name space.
-            newNameMap.put(newName, name);
-        }
-        else if (!name.equals(newNameMap.get(newName)))
-        {
-            // TODO: Preferentially keep the library method's name.
-            // TODO: Detect conflicting library member names.
-
-            // There's a conflict! A member (with a given old name) in a
-            // first namespace has received the same new name as this
-            // member (with a different old name) in a second name space,
-            // and now these two have to live together in this name space.
-
-            // Mark the name as conflicting.
-            MemberInfoNameConflictFilter.markConflictingName(memberInfo);
         }
     }
 
@@ -154,12 +142,11 @@ public class MemberInfoObfuscator implements MemberInfoVisitor
     // Small utility methods.
 
     /**
-     * Gets the [new name - old name] map, based on the given map
-     * [descriptor - new name - old name] and a given descriptor.
+     * Gets the name map, based on the given map and a given descriptor.
      * A new empty map is created if necessary.
      * @param descriptorMap the map of descriptors to [new name - old name] maps.
      * @param descriptor    the class member descriptor.
-     * @return the corresponding [new name - old name] map.
+     * @return the corresponding name map.
      */
     static Map retrieveNameMap(Map descriptorMap, String descriptor)
     {
@@ -211,7 +198,7 @@ public class MemberInfoObfuscator implements MemberInfoVisitor
 
     /**
      * Returns whether the new name of the given class member is fixed.
-     * @param memberInfo the given class member.
+     * @param memberInfo the class member.
      * @return whether its new name is fixed.
      */
     static boolean hasFixedNewMemberName(MemberInfo memberInfo)
